@@ -1,27 +1,13 @@
-/*=============================================================================
-
-    This file is part of ARB.
-
-    ARB is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    ARB is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with ARB; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
-
-=============================================================================*/
-/******************************************************************************
-
+/*
     Copyright (C) 2014 Fredrik Johansson
 
-******************************************************************************/
+    This file is part of Arb.
+
+    Arb is free software: you can redistribute it and/or modify it under
+    the terms of the GNU Lesser General Public License (LGPL) as published
+    by the Free Software Foundation; either version 2.1 of the License, or
+    (at your option) any later version.  See <http://www.gnu.org/licenses/>.
+*/
 
 #ifndef ARB_H
 #define ARB_H
@@ -32,7 +18,7 @@
 #define ARB_INLINE static __inline__
 #endif
 
-#include "fmprb.h"
+#include <stdio.h>
 #include "mag.h"
 #include "arf.h"
 
@@ -40,7 +26,8 @@
 extern "C" {
 #endif
 
-extern const char * arb_version;
+ARB_DLL extern const char * arb_version;
+double arb_test_multiplier(void);
 
 typedef struct
 {
@@ -67,47 +54,10 @@ arb_init(arb_t x)
     mag_init(arb_radref(x));
 }
 
-ARB_INLINE void
-arb_clear(arb_t x)
-{
-    arf_clear(arb_midref(x));
-    mag_clear(arb_radref(x));
-}
+void arb_clear(arb_t x);
 
-ARB_INLINE arb_ptr
-_arb_vec_init(slong n)
-{
-    slong i;
-    arb_ptr v = (arb_ptr) flint_malloc(sizeof(arb_struct) * n);
-
-    for (i = 0; i < n; i++)
-        arb_init(v + i);
-
-    return v;
-}
-
-ARB_INLINE void
-_arb_vec_clear(arb_ptr v, slong n)
-{
-    slong i;
-    for (i = 0; i < n; i++)
-        arb_clear(v + i);
-    flint_free(v);
-}
-
-ARB_INLINE void
-arb_set_fmprb(arb_t x, const fmprb_t y)
-{
-    arf_set_fmpr(arb_midref(x), fmprb_midref(y));
-    mag_set_fmpr(arb_radref(x), fmprb_radref(y));
-}
-
-ARB_INLINE void
-arb_get_fmprb(fmprb_t x, const arb_t y)
-{
-    arf_get_fmpr(fmprb_midref(x), arb_midref(y));
-    mag_get_fmpr(fmprb_radref(x), arb_radref(y));
-}
+arb_ptr _arb_vec_init(slong n);
+void _arb_vec_clear(arb_ptr v, slong n);
 
 ARB_INLINE arf_ptr arb_mid_ptr(arb_t z) { return arb_midref(z); }
 ARB_INLINE mag_ptr arb_rad_ptr(arb_t z) { return arb_radref(z); }
@@ -125,6 +75,13 @@ arb_equal(const arb_t x, const arb_t y)
            mag_equal(arb_radref(x), arb_radref(y));
 }
 
+ARB_INLINE int
+arb_equal_si(const arb_t x, slong y)
+{
+    return arf_equal_si(arb_midref(x), y) && mag_is_zero(arb_radref(x));
+}
+
+/* implementations are in arb/richcmp.c */
 int arb_eq(const arb_t x, const arb_t y);
 int arb_ne(const arb_t x, const arb_t y);
 int arb_lt(const arb_t x, const arb_t y);
@@ -132,12 +89,7 @@ int arb_le(const arb_t x, const arb_t y);
 int arb_gt(const arb_t x, const arb_t y);
 int arb_ge(const arb_t x, const arb_t y);
 
-ARB_INLINE void
-arb_zero(arb_t x)
-{
-    arf_zero(arb_midref(x));
-    mag_zero(arb_radref(x));
-}
+void arb_zero(arb_t x);
 
 ARB_INLINE int
 arb_is_zero(const arb_t x)
@@ -166,18 +118,9 @@ arb_zero_pm_inf(arb_t x)
     mag_inf(arb_radref(x));
 }
 
-ARB_INLINE void
-arb_indeterminate(arb_t x)
-{
-    arf_nan(arb_midref(x));
-    mag_inf(arb_radref(x));
-}
+void arb_indeterminate(arb_t x);
 
-ARB_INLINE int
-arb_is_finite(const arb_t x)
-{
-    return arf_is_finite(arb_midref(x)) && mag_is_finite(arb_radref(x));
-}
+int arb_is_finite(const arb_t x);
 
 void arb_set(arb_t x, const arb_t y);
 
@@ -199,6 +142,8 @@ void arb_neg_round(arb_t x, const arb_t y, slong prec);
 
 void arb_abs(arb_t y, const arb_t x);
 
+void arb_sgn(arb_t res, const arb_t x);
+
 void _arb_digits_round_inplace(char * s, mp_bitcnt_t * shift, fmpz_t error, slong n, arf_rnd_t rnd);
 
 int arb_set_str(arb_t res, const char * inp, slong prec);
@@ -217,33 +162,13 @@ arb_set_arf(arb_t x, const arf_t y)
     mag_zero(arb_radref(x));
 }
 
-ARB_INLINE void
-arb_set_si(arb_t x, slong y)
-{
-    arf_set_si(arb_midref(x), y);
-    mag_zero(arb_radref(x));
-}
+void arb_set_si(arb_t x, slong y);
 
-ARB_INLINE void
-arb_set_ui(arb_t x, ulong y)
-{
-    arf_set_ui(arb_midref(x), y);
-    mag_zero(arb_radref(x));
-}
+void arb_set_ui(arb_t x, ulong y);
 
-ARB_INLINE void
-arb_set_d(arb_t x, double y)
-{
-    arf_set_d(arb_midref(x), y);
-    mag_zero(arb_radref(x));
-}
+void arb_set_d(arb_t x, double y);
 
-ARB_INLINE void
-arb_set_fmpz(arb_t x, const fmpz_t y)
-{
-    arf_set_fmpz(arb_midref(x), y);
-    mag_zero(arb_radref(x));
-}
+void arb_set_fmpz(arb_t x, const fmpz_t y);
 
 ARB_INLINE void
 arb_set_fmpz_2exp(arb_t x, const fmpz_t y, const fmpz_t exp)
@@ -262,26 +187,34 @@ arb_is_one(const arb_t f)
     return arf_is_one(arb_midref(f)) && mag_is_zero(arb_radref(f));
 }
 
-ARB_INLINE void
-arb_one(arb_t f)
-{
-    arf_one(arb_midref(f));
-    mag_zero(arb_radref(f));
-}
+void arb_one(arb_t f);
 
-void arb_print(const arb_t x);
+void arb_fprint(FILE * file, const arb_t x);
 
-void arb_printd(const arb_t x, slong digits);
+void arb_fprintd(FILE * file, const arb_t x, slong digits);
 
-void arb_printn(const arb_t x, slong digits, ulong flags);
+void arb_fprintn(FILE * file, const arb_t x, slong digits, ulong flags);
 
 ARB_INLINE void
-arb_mul_2exp_si(arb_t y, const arb_t x, slong e)
+arb_print(const arb_t x)
 {
-    arf_mul_2exp_si(arb_midref(y), arb_midref(x), e);
-    mag_mul_2exp_si(arb_radref(y), arb_radref(x), e);
+    arb_fprint(stdout, x);
 }
 
+ARB_INLINE void
+arb_printd(const arb_t x, slong digits)
+{
+    arb_fprintd(stdout, x, digits);
+}
+
+ARB_INLINE void
+arb_printn(const arb_t x, slong digits, ulong flags)
+{
+    arb_fprintn(stdout, x, digits, flags);
+}
+
+void arb_mul_2exp_si(arb_t y, const arb_t x, slong e);
+
 ARB_INLINE void
 arb_mul_2exp_fmpz(arb_t y, const arb_t x, const fmpz_t e)
 {
@@ -297,80 +230,23 @@ arb_is_int(const arb_t x)
 }
 
 ARB_INLINE int
-arb_contains_zero(const arb_t x)
+arb_is_int_2exp_si(const arb_t x, slong e)
 {
-    return arf_cmpabs_mag(arb_midref(x), arb_radref(x)) <= 0;
+    return mag_is_zero(arb_radref(x)) &&
+           arf_is_int_2exp_si(arb_midref(x), e);
 }
 
-ARB_INLINE int
-arb_is_nonzero(const arb_t x)
-{
-    return !arb_contains_zero(x);
-}
-
-ARB_INLINE int
-arb_is_positive(const arb_t x)
-{
-    return (arf_sgn(arb_midref(x)) > 0) &&
-        (arf_mag_cmpabs(arb_radref(x), arb_midref(x)) < 0) &&
-         !arf_is_nan(arb_midref(x));
-}
-
-ARB_INLINE int
-arb_is_nonnegative(const arb_t x)
-{
-    return (arf_sgn(arb_midref(x)) >= 0) &&
-        (arf_mag_cmpabs(arb_radref(x), arb_midref(x)) <= 0) &&
-         !arf_is_nan(arb_midref(x));
-}
-
-ARB_INLINE int
-arb_is_negative(const arb_t x)
-{
-    return (arf_sgn(arb_midref(x)) < 0) &&
-        (arf_mag_cmpabs(arb_radref(x), arb_midref(x)) < 0) &&
-         !arf_is_nan(arb_midref(x));
-}
-
-ARB_INLINE int
-arb_is_nonpositive(const arb_t x)
-{
-    return (arf_sgn(arb_midref(x)) <= 0) &&
-        (arf_mag_cmpabs(arb_radref(x), arb_midref(x)) <= 0) &&
-         !arf_is_nan(arb_midref(x));
-}
-
-ARB_INLINE int
-arb_contains_negative(const arb_t x)
-{
-    return (arf_sgn(arb_midref(x)) < 0) ||
-        (arf_mag_cmpabs(arb_radref(x), arb_midref(x)) > 0)
-        || arf_is_nan(arb_midref(x));
-}
-
-ARB_INLINE int
-arb_contains_nonpositive(const arb_t x)
-{
-    return (arf_sgn(arb_midref(x)) <= 0) ||
-        (arf_mag_cmpabs(arb_radref(x), arb_midref(x)) >= 0)
-        || arf_is_nan(arb_midref(x));
-}
-
-ARB_INLINE int
-arb_contains_positive(const arb_t x)
-{
-    return (arf_sgn(arb_midref(x)) > 0) ||
-        (arf_mag_cmpabs(arb_radref(x), arb_midref(x)) > 0)
-        || arf_is_nan(arb_midref(x));
-}
-
-ARB_INLINE int
-arb_contains_nonnegative(const arb_t x)
-{
-    return (arf_sgn(arb_midref(x)) >= 0) ||
-        (arf_mag_cmpabs(arb_radref(x), arb_midref(x)) >= 0)
-        || arf_is_nan(arb_midref(x));
-}
+/* implementations are in arb/richcmp.c */
+int arb_contains_zero(const arb_t x);
+int arb_is_nonzero(const arb_t x);
+int arb_is_positive(const arb_t x);
+int arb_is_nonnegative(const arb_t x);
+int arb_is_negative(const arb_t x);
+int arb_is_nonpositive(const arb_t x);
+int arb_contains_negative(const arb_t x);
+int arb_contains_nonpositive(const arb_t x);
+int arb_contains_positive(const arb_t x);
+int arb_contains_nonnegative(const arb_t x);
 
 void arb_get_mag_lower(mag_t z, const arb_t x);
 
@@ -433,6 +309,26 @@ arb_get_abs_lbound_arf(arf_t u, const arb_t x, slong prec)
         arf_zero(u);
 }
 
+ARB_INLINE void
+arb_get_ubound_arf(arf_t u, const arb_t x, long prec)
+{
+    arf_t t;
+    arf_init_set_mag_shallow(t, arb_radref(x));
+
+    arf_add(u, arb_midref(x), t, prec, ARF_RND_CEIL);
+}
+
+ARB_INLINE void
+arb_get_lbound_arf(arf_t u, const arb_t x, long prec)
+{
+    arf_t t;
+    arf_init_set_mag_shallow(t, arb_radref(x));
+
+    arf_sub(u, arb_midref(x), t, prec, ARF_RND_FLOOR);
+}
+
+void arb_nonnegative_part(arb_t res, const arb_t x);
+
 slong arb_rel_error_bits(const arb_t x);
 
 ARB_INLINE slong
@@ -502,8 +398,15 @@ void arb_get_interval_arf(arf_t a, arf_t b, const arb_t x, slong prec);
 void arb_get_interval_mpfr(mpfr_t a, mpfr_t b, const arb_t x);
 
 void arb_union(arb_t z, const arb_t x, const arb_t y, slong prec);
+int arb_intersection(arb_t z, const arb_t x, const arb_t y, slong prec);
 void arb_get_rand_fmpq(fmpq_t q, flint_rand_t state, const arb_t x, slong bits);
 
+void arb_min(arb_t z, const arb_t x, const arb_t y, slong prec);
+void arb_max(arb_t z, const arb_t x, const arb_t y, slong prec);
+
+int arb_can_round_arf(const arb_t x, slong prec, arf_rnd_t rnd);
+int arb_can_round_mpfr(const arb_t x, slong prec, mpfr_rnd_t rnd);
+
 void arb_add(arb_t z, const arb_t x, const arb_t y, slong prec);
 void arb_add_arf(arb_t z, const arb_t x, const arf_t y, slong prec);
 void arb_add_ui(arb_t z, const arb_t x, ulong y, slong prec);
@@ -583,6 +486,7 @@ void arb_log_arf(arb_t z, const arf_t x, slong prec);
 void arb_log_ui(arb_t z, ulong x, slong prec);
 void arb_log_fmpz(arb_t z, const fmpz_t x, slong prec);
 void arb_log1p(arb_t r, const arb_t z, slong prec);
+void arb_log_base_ui(arb_t res, const arb_t x, ulong b, slong prec);
 void arb_exp(arb_t z, const arb_t x, slong prec);
 void arb_expm1(arb_t z, const arb_t x, slong prec);
 void arb_exp_invexp(arb_t z, arb_t w, const arb_t x, slong prec);
@@ -603,6 +507,7 @@ void arb_sin_cos_pi_fmpq(arb_t s, arb_t c, const fmpq_t x, slong prec);
 void arb_sin_pi_fmpq(arb_t s, const fmpq_t x, slong prec);
 void arb_cos_pi_fmpq(arb_t c, const fmpq_t x, slong prec);
 void arb_sinc(arb_t z, const arb_t x, slong prec);
+void arb_sinc_pi(arb_t z, const arb_t x, slong prec);
 void arb_sinh(arb_t z, const arb_t x, slong prec);
 void arb_cosh(arb_t z, const arb_t x, slong prec);
 void arb_sinh_cosh(arb_t s, arb_t c, const arb_t x, slong prec);
@@ -668,6 +573,9 @@ void arb_zeta_ui_vec_odd(arb_ptr x, ulong start, slong num, slong prec);
 void arb_zeta_ui_vec(arb_ptr x, ulong start, slong num, slong prec);
 void arb_bernoulli_ui(arb_t b, ulong n, slong prec);
 void arb_bernoulli_ui_zeta(arb_t b, ulong n, slong prec);
+void arb_bernoulli_fmpz(arb_t b, const fmpz_t n, slong prec);
+
+void arb_bernoulli_poly_ui(arb_t res, ulong n, const arb_t x, slong prec);
 
 void arb_polylog(arb_t w, const arb_t s, const arb_t z, slong prec);
 void arb_polylog_si(arb_t w, slong s, const arb_t z, slong prec);
@@ -682,6 +590,20 @@ void arb_bell_sum_taylor(arb_t res, const fmpz_t n, const fmpz_t a, const fmpz_t
 void arb_bell_sum_bsplit(arb_t res, const fmpz_t n, const fmpz_t a, const fmpz_t b, const fmpz_t mmag, slong prec);
 void arb_bell_fmpz(arb_t res, const fmpz_t n, slong prec);
 
+void arb_euler_number_fmpz(arb_t res, const fmpz_t n, slong prec);
+void arb_euler_number_ui(arb_t res, ulong n, slong prec);
+
+void arb_partitions_fmpz(arb_t res, const fmpz_t n, slong prec);
+void arb_partitions_ui(arb_t res, ulong n, slong prec);
+
+void arb_lambertw(arb_t res, const arb_t x, int flags, slong prec);
+
+ARB_INLINE void
+arb_sqr(arb_t res, const arb_t val, slong prec)
+{
+    arb_mul(res, val, val, prec);
+}
+
 #define ARB_DEF_CACHED_CONSTANT(name, comp_func) \
     TLS_PREFIX slong name ## _cached_prec = 0; \
     TLS_PREFIX arb_t name ## _cached_value; \
@@ -850,15 +772,6 @@ _arb_vec_dot(arb_t res, arb_srcptr vec1, arb_srcptr vec2, slong len2, slong prec
         arb_addmul(res, vec1 + i, vec2 + i, prec);
 }
 
-ARB_INLINE void
-_arb_vec_norm(arb_t res, arb_srcptr vec, slong len, slong prec)
-{
-    slong i;
-    arb_zero(res);
-    for (i = 0; i < len; i++)
-        arb_addmul(res, vec + i, vec + i, prec);
-}
-
 ARB_INLINE void
 _arb_vec_get_mag(mag_t bound, arb_srcptr vec, slong len)
 {
@@ -967,10 +880,10 @@ _arb_vec_get_unique_fmpz_vec(fmpz * res,  arb_srcptr vec, slong len)
 #define ARB_ATAN_TAB2_PREC 4608
 #define ARB_ATAN_TAB2_LIMBS (ARB_ATAN_TAB2_PREC / FLINT_BITS)
 
-extern const mp_limb_t arb_atan_tab1[1 << ARB_ATAN_TAB1_BITS][ARB_ATAN_TAB1_LIMBS];
-extern const mp_limb_t arb_atan_tab21[1 << ARB_ATAN_TAB21_BITS][ARB_ATAN_TAB2_LIMBS];
-extern const mp_limb_t arb_atan_tab22[1 << ARB_ATAN_TAB22_BITS][ARB_ATAN_TAB2_LIMBS];
-extern const mp_limb_t arb_atan_pi2_minus_one[ARB_ATAN_TAB2_LIMBS];
+ARB_DLL extern const mp_limb_t arb_atan_tab1[1 << ARB_ATAN_TAB1_BITS][ARB_ATAN_TAB1_LIMBS];
+ARB_DLL extern const mp_limb_t arb_atan_tab21[1 << ARB_ATAN_TAB21_BITS][ARB_ATAN_TAB2_LIMBS];
+ARB_DLL extern const mp_limb_t arb_atan_tab22[1 << ARB_ATAN_TAB22_BITS][ARB_ATAN_TAB2_LIMBS];
+ARB_DLL extern const mp_limb_t arb_atan_pi2_minus_one[ARB_ATAN_TAB2_LIMBS];
 
 void
 _arb_atan_taylor_naive(mp_ptr y, mp_limb_t * error,
@@ -991,11 +904,11 @@ void _arb_atan_taylor_rs(mp_ptr y, mp_limb_t * error,
 #define ARB_LOG_TAB2_PREC 4608
 #define ARB_LOG_TAB2_LIMBS (ARB_LOG_TAB2_PREC / FLINT_BITS)
 
-extern const mp_limb_t arb_log_tab11[1 << ARB_LOG_TAB11_BITS][ARB_LOG_TAB1_LIMBS];
-extern const mp_limb_t arb_log_tab12[1 << ARB_LOG_TAB12_BITS][ARB_LOG_TAB1_LIMBS];
-extern const mp_limb_t arb_log_tab21[1 << ARB_LOG_TAB21_BITS][ARB_LOG_TAB2_LIMBS];
-extern const mp_limb_t arb_log_tab22[1 << ARB_LOG_TAB22_BITS][ARB_LOG_TAB2_LIMBS];
-extern const mp_limb_t arb_log_log2_tab[ARB_LOG_TAB2_LIMBS];
+ARB_DLL extern const mp_limb_t arb_log_tab11[1 << ARB_LOG_TAB11_BITS][ARB_LOG_TAB1_LIMBS];
+ARB_DLL extern const mp_limb_t arb_log_tab12[1 << ARB_LOG_TAB12_BITS][ARB_LOG_TAB1_LIMBS];
+ARB_DLL extern const mp_limb_t arb_log_tab21[1 << ARB_LOG_TAB21_BITS][ARB_LOG_TAB2_LIMBS];
+ARB_DLL extern const mp_limb_t arb_log_tab22[1 << ARB_LOG_TAB22_BITS][ARB_LOG_TAB2_LIMBS];
+ARB_DLL extern const mp_limb_t arb_log_log2_tab[ARB_LOG_TAB2_LIMBS];
 
 /* exponential implementation */
 
@@ -1013,9 +926,9 @@ extern const mp_limb_t arb_log_log2_tab[ARB_LOG_TAB2_LIMBS];
 #define ARB_EXP_TAB2_PREC 4608
 #define ARB_EXP_TAB2_LIMBS (ARB_EXP_TAB2_PREC / FLINT_BITS)
 
-extern const mp_limb_t arb_exp_tab1[ARB_EXP_TAB1_NUM][ARB_EXP_TAB1_LIMBS];
-extern const mp_limb_t arb_exp_tab21[ARB_EXP_TAB21_NUM][ARB_EXP_TAB2_LIMBS];
-extern const mp_limb_t arb_exp_tab22[ARB_EXP_TAB22_NUM][ARB_EXP_TAB2_LIMBS];
+ARB_DLL extern const mp_limb_t arb_exp_tab1[ARB_EXP_TAB1_NUM][ARB_EXP_TAB1_LIMBS];
+ARB_DLL extern const mp_limb_t arb_exp_tab21[ARB_EXP_TAB21_NUM][ARB_EXP_TAB2_LIMBS];
+ARB_DLL extern const mp_limb_t arb_exp_tab22[ARB_EXP_TAB22_NUM][ARB_EXP_TAB2_LIMBS];
 
 void _arb_exp_taylor_naive(mp_ptr y, mp_limb_t * error,
     mp_srcptr x, mp_size_t xn, ulong N);
@@ -1052,12 +965,12 @@ void _arb_exp_sum_bs_simple(fmpz_t T, fmpz_t Q, mp_bitcnt_t * Qexp,
 #define ARB_SIN_COS_TAB2_PREC 4608
 #define ARB_SIN_COS_TAB2_LIMBS (ARB_SIN_COS_TAB2_PREC / FLINT_BITS)
 
-extern const mp_limb_t arb_sin_cos_tab1[2 * ARB_SIN_COS_TAB1_NUM][ARB_SIN_COS_TAB1_LIMBS];
-extern const mp_limb_t arb_sin_cos_tab21[2 * ARB_SIN_COS_TAB21_NUM][ARB_SIN_COS_TAB2_LIMBS];
-extern const mp_limb_t arb_sin_cos_tab22[2 * ARB_SIN_COS_TAB22_NUM][ARB_SIN_COS_TAB2_LIMBS];
+ARB_DLL extern const mp_limb_t arb_sin_cos_tab1[2 * ARB_SIN_COS_TAB1_NUM][ARB_SIN_COS_TAB1_LIMBS];
+ARB_DLL extern const mp_limb_t arb_sin_cos_tab21[2 * ARB_SIN_COS_TAB21_NUM][ARB_SIN_COS_TAB2_LIMBS];
+ARB_DLL extern const mp_limb_t arb_sin_cos_tab22[2 * ARB_SIN_COS_TAB22_NUM][ARB_SIN_COS_TAB2_LIMBS];
 
 #define ARB_PI4_TAB_LIMBS (4608 / FLINT_BITS)
-extern const mp_limb_t arb_pi4_tab[ARB_PI4_TAB_LIMBS];
+ARB_DLL extern const mp_limb_t arb_pi4_tab[ARB_PI4_TAB_LIMBS];
 
 void _arb_sin_cos_taylor_naive(mp_ptr ysin, mp_ptr ycos, mp_limb_t * error,
     mp_srcptr x, mp_size_t xn, ulong N);
@@ -1103,6 +1016,38 @@ void _arb_atan_sum_bs_powtab(fmpz_t T, fmpz_t Q, mp_bitcnt_t * Qexp,
 
 void arb_atan_arf_bb(arb_t z, const arf_t x, slong prec);
 
+ARB_INLINE slong
+arb_allocated_bytes(const arb_t x)
+{
+    return arf_allocated_bytes(arb_midref(x)) + mag_allocated_bytes(arb_radref(x));
+}
+
+ARB_INLINE slong
+_arb_vec_allocated_bytes(arb_srcptr vec, slong len)
+{
+    slong i, size;
+
+    size = len * sizeof(arb_struct);
+
+    for (i = 0; i < len; i++)
+        size += arb_allocated_bytes(vec + i);
+
+    return size;
+}
+
+ARB_INLINE double
+_arb_vec_estimate_allocated_bytes(slong len, slong prec)
+{
+    double size;
+
+    size = len * (double) sizeof(arb_struct);
+
+    if (prec > ARF_NOPTR_LIMBS * FLINT_BITS)
+        size += len * (double) ((prec + FLINT_BITS - 1) / FLINT_BITS) * sizeof(mp_limb_t);
+
+    return size;
+}
+
 #ifdef __cplusplus
 }
 #endif
